X-Git-Url: http://cvs.zerfleddert.de/cgi-bin/gitweb.cgi/proxmark3-svn/blobdiff_plain/5f91a683b7f3b3b9217db9407a13b1d738070f01..6e2f51a06e981bb8ff99c02aff2c5a5ffc0ebbf9:/client/proxmark3.c?ds=inline

diff --git a/client/proxmark3.c b/client/proxmark3.c
index ef3d33e8..528cae34 100644
--- a/client/proxmark3.c
+++ b/client/proxmark3.c
@@ -23,10 +23,15 @@
 #include "uart.h"
 #include "ui.h"
 #include "sleep.h"
+#include "cmdparser.h"
+#include "cmdmain.h"
+
+// a global mutex to prevent interlaced printing from different threads
+pthread_mutex_t print_lock;
 
 static serial_port sp;
 static UsbCommand txcmd;
-static bool txcmd_pending = false;
+volatile static bool txcmd_pending = false;
 
 void SendCommand(UsbCommand *c) {
 #if 0
@@ -37,6 +42,12 @@ void SendCommand(UsbCommand *c) {
     ERR("Sending command failed, previous command is still pending");
   }
 */
+  if(offline)
+    {
+      PrintAndLog("Sending bytes to proxmark failed - offline");
+      return;
+    }
+
   while(txcmd_pending);
   txcmd = *c;
   txcmd_pending = true;
@@ -164,6 +175,7 @@ static void *main_loop(void *targ) {
 			
 			if (cmd[0] != 0x00) {
 				if (strncmp(cmd, "quit", 4) == 0) {
+					exit(0);
 					break;
 				}
 				
@@ -195,21 +207,52 @@ static void *main_loop(void *targ) {
   return NULL;
 }
 
+//static void dumpHelp(char  *parent, ...)
+//{
+//  printf("## %s\n\n", parent);
+//  CommandReceived(parent);
+//  
+//  printf("\n");
+//}
+
+static void dumpAllHelp(int markdown)
+{
+  printf("\n%sProxmark3 command dump%s\n\n",markdown?"# ":"",markdown?"":"\n======================");
+  printf("Some commands are available only if a Proxmark is actually connected.%s\n",markdown?"  ":"");
+  printf("Check column \"offline\" for their availability.\n");
+  printf("\n");
+  command_t *cmds = getTopLevelCommandTable();
+  dumpCommandsRecursive(cmds, markdown);
+}
+
 int main(int argc, char* argv[]) {
-  srand(time(0));
-  
-  if (argc < 2) {
-    printf("syntax: %s <port>\n\n",argv[0]);
-    printf("\tLinux example:'%s /dev/ttyACM0'\n\n", argv[0]);
-    return 1;
-  }
+	srand(time(0));
   
-  // Make sure to initialize
-  struct main_loop_arg marg = {
-    .usb_present = 0,
-    .script_cmds_file = NULL
-  };
-  pthread_t main_loop_t;
+	if (argc < 2) {
+		printf("syntax: %s <port>\n\n",argv[0]);
+		printf("\tLinux example:'%s /dev/ttyACM0'\n\n", argv[0]);
+		printf("help:   %s -h\n\n", argv[0]);
+		printf("\tDump all interactive help at once\n");
+		printf("markdown:   %s -m\n\n", argv[0]);
+		printf("\tDump all interactive help at once in markdown syntax\n");
+		return 1;
+	}
+	if (strcmp(argv[1], "-h") == 0) {
+		printf("syntax: %s <port>\n\n",argv[0]);
+		printf("\tLinux example:'%s /dev/ttyACM0'\n\n", argv[0]);
+		dumpAllHelp(0);
+		return 0;
+	}
+	if (strcmp(argv[1], "-m") == 0) {
+		dumpAllHelp(1);
+		return 0;
+	}
+	// Make sure to initialize
+	struct main_loop_arg marg = {
+		.usb_present = 0,
+		.script_cmds_file = NULL
+	};
+	pthread_t main_loop_t;
 
 /*
   usb_init();
@@ -223,38 +266,54 @@ int main(int argc, char* argv[]) {
   }
 */
   
-  sp = uart_open(argv[1]);
-  if (sp == INVALID_SERIAL_PORT) {
-    printf("ERROR: invalid serial port\n");
-    marg.usb_present = 0;
-    offline = 1;
-  } else if (sp == CLAIMED_SERIAL_PORT) {
-    printf("ERROR: serial port is claimed by another process\n");
-    marg.usb_present = 0;
-    offline = 1;
-  } else {
-    marg.usb_present = 1;
-    offline = 0;
-  }
+	sp = uart_open(argv[1]);
+	if (sp == INVALID_SERIAL_PORT) {
+		printf("ERROR: invalid serial port\n");
+		marg.usb_present = 0;
+		offline = 1;
+	} else if (sp == CLAIMED_SERIAL_PORT) {
+		printf("ERROR: serial port is claimed by another process\n");
+		marg.usb_present = 0;
+		offline = 1;
+	} else {
+		marg.usb_present = 1;
+		offline = 0;
+	}
 
-  // If the user passed the filename of the 'script' to execute, get it
-  if (argc > 2 && argv[2]) {
-    marg.script_cmds_file = argv[2];
-  }
-  
-  pthread_create(&main_loop_t, NULL, &main_loop, &marg);
-  InitGraphics(argc, argv);
+	// If the user passed the filename of the 'script' to execute, get it
+	if (argc > 2 && argv[2]) {
+		if (argv[2][0] == 'f' &&  //buzzy, if a word 'flush' passed, flush the output after every log entry.
+			argv[2][1] == 'l' &&
+			argv[2][2] == 'u' &&
+			argv[2][3] == 's' &&
+			argv[2][4] == 'h')
+		{
+			printf("Output will be flushed after every print.\n");
+			flushAfterWrite = 1;
+		}
+		else
+		marg.script_cmds_file = argv[2];
+	}
 
-  MainGraphics();
+	// create a mutex to avoid interlacing print commands from our different threads
+	pthread_mutex_init(&print_lock, NULL);
 
-  pthread_join(main_loop_t, NULL);
+	pthread_create(&main_loop_t, NULL, &main_loop, &marg);
+	InitGraphics(argc, argv);
+
+	MainGraphics();
+
+	pthread_join(main_loop_t, NULL);
 
 //  if (marg.usb_present == 1) {
 //    CloseProxmark();
 //  }
 
-  // Clean up the port
-  uart_close(sp);
+	// Clean up the port
+	uart_close(sp);
+  
+	// clean up mutex
+	pthread_mutex_destroy(&print_lock);
   
   return 0;
 }